<?php
// $Horde: turba/lib/Driver/sql.php,v 1.5 2001/06/02 23:25:20 jon Exp $

/**
 * Turba directory driver implementation for PHP's PEAR database abstraction
 * layer.
 *
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.5 $
 * @since   Turba 0.0.1
 * @package turba
 */
class Turba_Driver_sql extends Turba_Driver {
    
    /** Handle for the current database connection. */
    var $db = null;
    
    /** String containg the name of the table in which the entries are
        stored. */
    var $table = '';
    
    
    /**
     * Constructs a new Turba SQL driver object.
     *
     * @param $params       Hash containing additional configuration parameters.
     */
    function Turba_Driver_sql($params)
    {
        $this->type = 'sql';
        $this->params = $params;
        
        include_once 'DB.php';
        $this->db = &DB::connect($params, true);
        $this->table = $params['table'];
    }
    
    
    /**
     * Searches the SQL database with the given criteria and returns a
     * filtered list of results.
     *
     * @param $criteria      Array containing the search criteria.
     * @param $fields        List of fields to return.
     * @param $strict_fields (optional) Array of fields which must be matched
     *                       exactly.
     *
     * @return               Hash containing the search results.
     */
    function search($criteria, $fields, $strict_fields = array())
    {
        $where = '';
        foreach ($criteria as $key => $val) {
            if (!empty($where)) $where .= ' and ';
            if (in_array($key, $strict_fields)) {
                $where .= "$key = '$val'";
            } else {
                $where .= "$key like '%$val%'";
            }
        }
        
        $query  = 'SELECT ' . implode(', ', $fields) . ' ';
        $query .= 'FROM ' . $this->table . ' WHERE ' . $where;
        
        $result = $this->db->query($query);
        
        $results = array();
        if (!DB::isError($result)) {
            while ($row = $result->fetchRow()) {
                if (DB::isError($row)) {
                    // error handling
                    break;
                }
                
                $entry = array();
                
                for ($i = 0; $i < count($fields); $i++) {
                    $field = $fields[$i];
                    $entry[$field] = $row[$i];
                }
                
                $results[] = $entry;
            }
        }
        
        return $results;
    }
    
    /**
     * Read the given data from the SQL database and returns
     * the result's fields.
     *
     * @param $criteria      Search criteria.
     * @param $id            Data identifier.
     * @param $fields        List of fields to return.
     *
     * @return               Hash containing the search results.
     */
    function read($criteria, $id, $fields)
    {
        $where = $criteria . " = '" . $this->db->quoteString($id) . "'";
        
        $query  = 'SELECT ' . implode(', ', $fields) . ' ';
        $query .= 'FROM ' . $this->table . ' WHERE ' . $where;
        
        $result = $this->db->query($query);
        
        $results = array();
        if (!DB::isError($result)) {
            $row = $result->fetchRow();
            if (isset($row) && !DB::isError($row)) {
                $entry = array();
                
                for ($i = 0; $i < count($fields); $i++) {
                    $field = $fields[$i];
                    $entry[$field] = $row[$i];
                }
                
                $results[] = $entry;
            }
        }
        
        return $results;    
    }
    
    /**
     * Closes the current database connection.
     *
     * @return          The result of the disconnect() call.
     */
    function close()
    {
        return $this->db->disconnect();
    }
    
    /**
     * Adds the specified object to the SQL database.
     */
    function addObject($attributes)
    {
        $fields = array();
        $values = array();
        foreach ($attributes as $field => $value) {
            $fields[] = $field;
            $values[] = "'" . $this->db->quoteString($value) . "'";
        }
        
        $query  = 'INSERT INTO ' . $this->table . ' (' . implode(', ', $fields) . ')';
        $query .= ' VALUES (' . implode(', ', $values) . ')';
        
        $result = $this->db->query($query);
        
        return (!DB::isError($result));
    }
    
    /**
     * Deletes the specified object from the SQL database.
     */
    function removeObject($object_key, $object_id)
    {
        $where = $object_key . " = '" . $this->db->quoteString($object_id) . "'";
        $query = 'DELETE FROM ' . $this->table . ' WHERE ' . $where;
    
        $result = $this->db->query($query);
        
        return (!DB::isError($result));
    }
    
    /**
     * Saves the specified object in the SQL database.
     */
    function setObject($object_key, $object_id, $attributes)
    {
        $where = $object_key . " = '" . $this->db->quoteString($object_id) . "'";
        unset($attributes[$object_key]);
        
        $set = array();
        foreach ($attributes as $field => $value) {
            $set[] = $field . " = '" . $this->db->quoteString($value) . "'";
        }
        
        $query  = 'UPDATE ' . $this->table . ' SET ' . implode(', ', $set) . ' ';
        $query .= 'WHERE ' . $where;
        
        $result = $this->db->query($query);
        
        return (!DB::isError($result));
    }
    
    /**
     * Create an object key for a new object.
     *
     * @param array $attributes  The attributes (in driver keys) of the
     *                           object being added.
     *
     * @return string  A unique ID for the new object.
     */
    function makeKey($attributes)
    {
        return md5(uniqid(rand()));
    }
    
}
?>
